;; Generated package description from expand-region.el  -*- no-byte-compile: t -*-
(define-package "expand-region" "1.0.0" "Increase selected region by semantic units." '((emacs "24.4")) :commit "9e3f86c02c5e2ab6f0d95da8a34045b54f6166d1" :authors '(("Magnar Sveen" . "magnars@gmail.com")) :maintainer '("Magnar Sveen" . "magnars@gmail.com") :keywords '("marking" "region") :url "https://github.com/magnars/expand-region.el")
